#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <glib.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <attr/xattr.h>
#include <we_bdf.h>

// Switch for whether or not to use extended file system attributes
// for BDF metadata. TODO: make this an autoconf switch
#define USE_XATTRS

static void parse_command_line(gint argc, gchar *argv[], gchar **bdf_filename);

gint
main(gint argc, gchar *argv[])
{
    gint result = EXIT_FAILURE;
    gchar *bdf_filename = NULL;
    FILE *bdf_file = NULL;

    parse_command_line(argc, argv, &bdf_filename);
    if (bdf_filename == NULL) {
        g_printerr("Usage: %s [BDF]\n", g_path_get_basename(argv[0]));
        goto cleanup_and_return;
    }

    bdf_file = fopen(bdf_filename, "r+");
    if (bdf_file == NULL) {
        g_printerr("Failed to open BDF file '%s': %s\n",
                   bdf_filename, g_strerror(errno));
        goto cleanup_and_return;
    }

    we_bdf_t *bdf = we_bdf_read(bdf_file);
    if (bdf == NULL) {
        g_printerr("Failed to read BDF file '%s'\n", bdf_filename);
        goto cleanup_and_return;
    }
    we_bdf_close(bdf);
    gchar **subsets = we_bdf_project_paths(bdf);
#ifdef USE_XATTRS
    gchar *integrations = g_strdup_printf("%u", g_strv_length(subsets));
    gint fd = fileno(bdf_file);
    if (fd == -1) {
        g_warning("Failed to get file descriptor: %s", g_strerror(errno));
    }
    gint rc = fsetxattr(fd, "user.edu.nrao.widar.integrations",
                        integrations, strlen(integrations), XATTR_REPLACE);
    if (rc == -1 && errno != ENOATTR) {
        g_printerr("Failed to set user.edu.nrao.widar.integrations attribute "
                   "to %s on %s", integrations, bdf_filename);
    }
    g_free(integrations);
#else
    printf("%u", g_strv_length(subsets));
#endif /* USE_XATTRS */
    g_strfreev(subsets);
    we_bdf_free(bdf);
    result = EXIT_SUCCESS;

 cleanup_and_return:
    if (bdf_file != NULL) {
        fclose(bdf_file);
    }
    if (bdf_filename != NULL) {
        g_free(bdf_filename);
    }
    return result;
}

static void
parse_command_line(gint argc, gchar *argv[], gchar **bdf_filename)
{
    if (argc == 2) {
        *bdf_filename = g_strdup(argv[1]);
    }
}
