#ifndef TERM_SOURCE_H_
#define TERM_SOURCE_H_
#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <glib.h>
#include <time.h>

G_BEGIN_DECLS

/**
 * TermSource:
 * @gsource: #GSource "base class" instance
 * @sig_timestamp: timestamp of last signal that was caught
 *
 * GLib event source to handle termination signal events. Structure
 * extends #GSource. All fields are private.
 */
typedef struct TermSource {
    /* <private> */
    GSource gsource;
    struct timespec sig_timestamp;
} TermSource;

/**
 * TermSourceFunc:
 *
 * Callback function type for a #TermSource. The argument to the
 * callback is a pointer to the value of the most recently handled
 * signal.
 */
typedef gboolean (*TermSourceFunc)(gint *sig);

TermSource *term_source_new(void);
void term_source_set_callback(TermSource *source, TermSourceFunc func);

// Note: most glib functions taking a GSource argument may be used
// with a simple typecast from a TermSource pointer to a GSource
// pointer
// OK, to save a little typing down the road...
/**
 * term_source_ref:
 * @t: #TermSource
 *
 * Increment reference count of @t.
 *
 * @see_also: g_source_ref()
 */
#define term_source_ref(t) g_source_ref((GSource*)(t))

/**
 * term_source_unref:
 * @t: #TermSource
 *
 * Decrement reference count of @t.
 *
 * @see_also: g_source_unref()
 */
#define term_source_unref(t) g_source_unref((GSource*)(t))

/**
 * term_source_attach:
 * @t: #TermSource
 * @c:context
 *
 * Add a source to a main loop context.
 *
 * @see_also: g_source_attach()
 */
#define term_source_attach(t, c) g_source_attach((GSource*)(t), (c))

/**
 * term_source_destroy:
 * @t: #TermSource
 *
 * Destroy source.
 *
 * @see_also: g_source_destroy()
 */
#define term_source_destroy(t) g_source_destroy((GSource*)(t))

G_END_DECLS
#endif /* TERM_SOURCE_H_ */
