#ifndef WCBE_PRIVATE_H_
#define WCBE_PRIVATE_H_
#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <glib.h>
#include <gst/gst.h>

#include <wcbe.h>
#include <wcbe_app.h>
#include <wcbe_lag_frame.h>
#include <wcbe_config.h>
#include <enums.h>
#include <sys/un.h>

G_BEGIN_DECLS

typedef enum wcbe_pipeline_state_t {
    WCBE_STATE_INITIALIZING, WCBE_STATE_RUNNING, WCBE_STATE_FINALIZING,
    WCBE_STATE_EXITING
} wcbe_pipeline_state_t;

typedef struct input_stage_t {
    guint bus_watch;
    GIOChannel *lsp_channel;
    guint lsp_source;
    gint lsp_sockfd;
    GstElement *pipeline;
    gboolean received_eos;
} input_stage_t;

typedef struct pipeline {
    GMainLoop *loop;
    gchar *name;
    gchar *src;
    gchar *ts0;
    wcbe_dump_format_t framedump;
    wcbe_flow_t flow;
    guint fps;
    gchar *sockdir;
    gchar *pidsdir;
    GIOChannel *ctlsock;
#if CONTROL_SOCK_TCP
    guint ctlsock_source;
    GIOChannel *listensock;
#endif
    gchar *auxdir;
    gchar *bdfdir;
    gchar *srcpath;
    input_stage_t current;
    input_stage_t old;
    GHashTable *configs;
    GStaticRecMutex configs_mtx;
    GSList *configless_pipelines;
    GSList *activations;
    GHashTable *msg_callbacks;
    guint bus_watch;
    wcbe_pipeline_state_t state;
#if CONTROL_SOCK_UDP || CONTROL_SOCK_SCTP || CONTROL_SOCK_TCP
    gchar *ctladdr;
#endif
} pipeline_t;

typedef enum {CONFIG_STATE_INITIALIZING=0, CONFIG_STATE_READY,
              CONFIG_STATE_NOT_READY, CONFIG_STATE_FINALIZING} config_state_t;

typedef struct configuration {
    wcbe_config_t wconfig;
    guint num_activations;
    GSList *ls_pipelines;
    gboolean destroy;
    pipeline_t *pipeline;
    config_state_t state;
} configuration_t;

typedef struct ls_pipeline {
    gchar *name;
    struct sockaddr_un sockaddr;
    gchar *sockname;
    GPid pid;
    gchar **argv;
    guint spawn_count;
    pipeline_t *pipeline;
    configuration_t *config;
    guint exit_timeout;
} ls_pipeline_t;

typedef struct component_pipeline {
    enum {INPUT_PIPELINE, LAGSET_PIPELINE} type;
    ls_pipeline_t lsp;
    pipeline_state_t state;
    gboolean destroy;
} component_pipeline_t;

extern gboolean wcbe_create_config(pipeline_t *pipeline,
                                   configuration_t *config);
extern void wcbe_insert_pending_conf(pipeline_t *pipeline, guint action,
                                     configuration_t *config);
extern void wcbe_cleanup_config(configuration_t *config);
extern void wcbe_add_activation(
    pipeline_t *pipeline, configuration_t *config, const lf_widar_time_t *time,
    gboolean now);
extern void wcbe_remove_activation(pipeline_t *pipeline,
                                   configuration_t *config,
                                   const lf_widar_time_t *time);
extern gboolean wcbe_delayed_activate_config(gpointer data);
extern void wcbe_destroy_config(pipeline_t *pipeline, configuration_t *config);
extern void wcbe_destroy_pipeline(pipeline_t *pipeline);
extern void wcbe_create_pipeline(pipeline_t *pipeline);
extern GstStateChangeReturn wcbe_start_pipeline(
    pipeline_t *pipeline, gboolean in_main_loop, GstClockTime timeout);
extern GstStateChangeReturn wcbe_stop_pipeline(
    pipeline_t *pipeline, gboolean in_main_loop, GstClockTime timeout);
extern component_pipeline_t *wcbe_find_component_pipeline(
    GSList *list, const gchar *name);
extern gchar *wcbe_parse_ts0(const gchar *value);

#define DELAYED_ACTIVATE_INTERVAL 250

#define DEFAULT_PIPELINE_NAME NULL
#define DEFAULT_PIPELINE_SRC NULL
#define DEFAULT_PIPELINE_CONFIGS NULL
#define DEFAULT_PIPELINE_CONFIG NULL
#define DEFAULT_PIPELINE_TS0 NULL
#define DEFAULT_PIPELINE_FRAMEDUMP WCBE_DUMP_NONE
#define DEFAULT_PIPELINE_FLOW WCBE_FLOW_OFF
#define DEFAULT_PIPELINE_FPS 100
#define DEFAULT_PIPELINE_SOCKDIR NULL
#define DEFAULT_PIPELINE_LSP_CHANNEL NULL
#define DEFAULT_PIPELINE_MSG_CALLBACKS NULL

extern gchar const input_stage_element_name[];
extern gchar const src_element_name[];
extern gchar const framedump_element_name[];
extern gchar const decode_element_name[];
extern gchar const sort_element_name[];
extern gchar const blsdump_element_name[];
extern gchar const sink_element_name[];
extern gchar const input_stage_pipeline_name[];


extern guint create_cfg_signal;
extern guint destroy_cfg_signal;
extern guint add_actvn_signal;
extern guint remove_actvn_signal;
extern guint new_subscan_signal;
extern guint cancel_subscan_signal;

G_END_DECLS

#endif /* WCBE_PRIVATE_H_ */
