#ifndef COBDF_H_
#define COBDF_H_
#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <glib.h>
#include <glib-object.h>
#include <complex.h>

#define USE_ALMA_5_1_0_5_XML_WORKAROUNDS
#define USE_ALMA_5_1_0_5_MIME_WORKAROUNDS
#define USE_BDF_V1_0_SCHEMA_WORKAROUNDS

G_BEGIN_DECLS

/**
 * CobdfAxisName:
 * @COBDF_INVALID_AXIS: "not-an-axis" value
 * @COBDF_POLARIZATION_AXIS: polarization product axis
 * @COBDF_SPECTRAL_POINT_AXIS: spectral point (or channel) axis
 * @COBDF_ATM_PHASE_CORRECTION_AXIS: atmospheric phase correction axis
 * @COBDF_BIN_AXIS: bin axis
 * @COBDF_SPECTRAL_WINDOW_AXIS: spectral window axis
 * @COBDF_BASEBAND_AXIS: baseband axis
 * @COBDF_ANTENNA_AXIS: antenna axis
 * @COBDF_BASELINE_AXIS: baseline axis
 * @COBDF_TIME_AXIS: time axis
 * @COBDF_NUM_AXES: number of axes
 *
 * Binary component axis names. These are ordered in increasing tree
 * level order.
 */
typedef enum {
    COBDF_INVALID_AXIS = -1,          /*< nick=invalid-axis >*/
    COBDF_POLARIZATION_AXIS = 0,      /*< nick=POL >*/
    COBDF_SPECTRAL_POINT_AXIS,        /*< nick=SPP >*/
    COBDF_ATM_PHASE_CORRECTION_AXIS,  /*< nick=APC >*/
    COBDF_BIN_AXIS,                   /*< nick=BIN >*/
    COBDF_SPECTRAL_WINDOW_AXIS,       /*< nick=SPW >*/
    COBDF_BASEBAND_AXIS,              /*< nick=BAB >*/
    COBDF_ANTENNA_AXIS,               /*< nick=ANT >*/
    COBDF_BASELINE_AXIS,              /*< nick=BAL >*/
    COBDF_TIME_AXIS,                  /*< nick=TIM >*/
    COBDF_NUM_AXES                    /*< nick=num-axes >*/
} CobdfAxisName;
extern GType cobdf_axis_name_enum_get_type(void);
#define COBDF_AXIS_NAME_ENUM_TYPE cobdf_axis_name_enum_get_type()

/**
 * CobdfPrimitiveDataType:
 * @COBDF_INT16: 16-bit twos-complement integer
 * @COBDF_INT32: 32-bit twos-complement integer
 * @COBDF_INT64: 64-bit twos-complement integer
 * @COBDF_FLOAT32: IEEE 754 single format floating point number (32-bit)
 *
 * Primitive data types.
 */
typedef enum {
    COBDF_INT16,   /*< nick=INT16_TYPE >*/
    COBDF_INT64,   /*< nick=INT64_TYPE >*/
    COBDF_INT32,   /*< nick=INT32_TYPE >*/
    COBDF_FLOAT32, /*< nick=FLOAT32_TYPE >*/
#ifdef USE_ALMA_5_1_0_5_XML_WORKAROUNDS
    COBDF_INT,     /*< nick=INT_TYPE >*/
    COBDF_FLOAT    /*< nick=FLOAT_TYPE >*/
#endif
} CobdfPrimitiveDataType;
extern GType cobdf_primitive_data_type_enum_get_type(void);
#define COBDF_PRIMITIVE_DATA_TYPE_ENUM_TYPE \
        cobdf_primitive_data_type_enum_get_type()

/**
 * CobdfAtmPhaseCorrection:
 * @COBDF_AP_INVALID: "not-an-apc-value" value
 * @COBDF_AP_UNCORRECTED: uncorrected
 * @COBDF_AP_CORRECTED: corrected
 * @COBDF_AP_MIXED: mixed
 *
 * Atmospheric phase correction values.
 */
typedef enum {
    COBDF_AP_INVALID = -1, /*< nick=invalid-apc >*/
    COBDF_AP_UNCORRECTED,  /*< nick=AP_UNCORRECTED >*/
    COBDF_AP_CORRECTED,    /*< nick=AP_CORRECTED >*/
    COBDF_AP_MIXED         /*< nick=AP_MIXED >*/
} CobdfAtmPhaseCorrection;
extern GType cobdf_atm_phase_correction_enum_get_type(void);
#define COBDF_ATM_PHASE_CORRECTION_ENUM_TYPE \
        cobdf_atm_phase_correction_enum_get_type()

/**
 * CobdfByteOrder:
 * @COBDF_LITTLE_ENDIAN: little endian
 * @COBDF_BIG_ENDIAN: big endian
 * @COBDF_INVALID_ENDIAN: "not-a-byte-order" value
 *
 * Byte order. Values are in agreement with GLib macros.
 */
typedef enum {
    COBDF_LITTLE_ENDIAN = G_LITTLE_ENDIAN, /*< nick=Little_Endian >*/
    COBDF_BIG_ENDIAN = G_BIG_ENDIAN,       /*< nick=Big_Endian >*/
    COBDF_INVALID_ENDIAN = 0               /*< nick=invalid-endian >*/
} CobdfByteOrder;
extern GType cobdf_byte_order_enum_get_type(void);
#define COBDF_BYTE_ORDER_ENUM_TYPE cobdf_byte_order_enum_get_type()

/**
 * CobdfBincomp:
 * @COBDF_ACTUAL_TIMES_BINCOMP: ACTUAL_TIMES
 * @COBDF_ACTUAL_DURATIONS_BINCOMP: ACTUAL_DURATIONS
 * @COBDF_ZERO_LAGS_BINCOMP: ZERO_LAGS
 * @COBDF_FLAGS_BINCOMP: FLAGS
 * @COBDF_CROSS_DATA_BINCOMP: CROSS_DATA
 * @COBDF_AUTO_DATA_BINCOMP: AUTO_DATA
 * @COBDF_WEIGHTS_BINCOMP: WEIGHTS
 * @COBDF_NUM_BINCOMPS: number of binary component types
 *
 * BDF binary component types (these should be consistently referred
 * to as "bincomps".
 */
typedef enum {
    COBDF_ACTUAL_TIMES_BINCOMP = 0,
    COBDF_ACTUAL_DURATIONS_BINCOMP,
    COBDF_ZERO_LAGS_BINCOMP,
    COBDF_FLAGS_BINCOMP,
    COBDF_CROSS_DATA_BINCOMP,
    COBDF_AUTO_DATA_BINCOMP,
    COBDF_WEIGHTS_BINCOMP,
    COBDF_NUM_BINCOMPS
} CobdfBincomp;
extern GType cobdf_bincomp_enum_get_type(void);
#define COBDF_BINCOMP_ENUM_TYPE cobdf_bincomp_enum_get_type()

/**
 * CobdfCorrelationMode:
 * @COBDF_CROSS_ONLY: CROSS_ONLY
 * @COBDF_AUTO_ONLY: AUTO_ONLY
 * @COBDF_CROSS_AND_AUTO: CROSS_AND_AUTO
 *
 * BDF CorrelationMode values.
 */
typedef enum {
    COBDF_CROSS_ONLY,     /*< nick=CROSS_ONLY >*/
    COBDF_AUTO_ONLY,      /*< nick=AUTO_ONLY >*/
    COBDF_CROSS_AND_AUTO  /*< nick=CROSS_AND_AUTO >*/
} CobdfCorrelationMode;
extern GType cobdf_correlation_mode_enum_get_type(void);
#define COBDF_CORRELATION_MODE_ENUM_TYPE cobdf_correlation_mode_enum_get_type()

/**
 * CobdfSpectralResolutionType:
 * @COBDF_FULL_RESOLUTION: FULL_RESOLUTION
 * @COBDF_CHANNEL_AVERAGE: CHANNEL_AVERAGE
 * @COBDF_BASEBAND_WIDE: BASEBAND_WIDE
 * @COBDF_RESOLUTION_UNKNOWN: unspecified SpectralResolutionType
 *
 * BDF SpectralResolutionType values.
 */
typedef enum {
    COBDF_FULL_RESOLUTION,    /*< nick=FULL_RESOLUTION >*/
    COBDF_CHANNEL_AVERAGE,    /*< nick=CHANNEL_AVERAGE >*/
    COBDF_BASEBAND_WIDE,      /*< nick=BASEBAND_WIDE >*/
    COBDF_RESOLUTION_UNKNOWN  /*< nick=resolution-unknown >*/
} CobdfSpectralResolutionType;
extern GType cobdf_spectral_resolution_type_enum_get_type(void);
#define COBDF_SPECTRAL_RESOLUTION_TYPE_ENUM_TYPE \
        cobdf_spectral_resolution_type_enum_get_type()

/**
 * CobdfNetSideband:
 * @COBDF_NOSB: NOSB
 * @COBDF_USB: USB
 * @COBDF_LSB: LSB
 * @COBDF_DSB: DSB
 *
 * BDF NetSideband values.
 */
typedef enum {
    COBDF_NOSB,  /*< nick=NOSB >*/
    COBDF_USB,   /*< nick=USB >*/
    COBDF_LSB,   /*< nick=LSB >*/
    COBDF_DSB    /*< nick=DSB >*/
} CobdfNetSideband;
extern GType cobdf_net_sideband_enum_get_type(void);
#define COBDF_NET_SIDEBAND_ENUM_TYPE cobdf_net_sideband_enum_get_type()

/**
 * CobdfStokesParameter:
 * @COBDF_RR: RR
 * @COBDF_RL: RL
 * @COBDF_LR: LR
 * @COBDF_LL: LL
 * @COBDF_XX: XX
 * @COBDF_XY: XY
 * @COBDF_YX: YX
 * @COBDF_YY: YY
 *
 * Stokes parameter values.
 */
typedef enum {
    COBDF_RR,  /*< nick=RR >*/
    COBDF_RL,  /*< nick=RL >*/
    COBDF_LR,  /*< nick=LR >*/
    COBDF_LL,  /*< nick=LL >*/
    COBDF_XX,  /*< nick=XX >*/
    COBDF_XY,  /*< nick=XY >*/
    COBDF_YX,  /*< nick=YX >*/
    COBDF_YY   /*< nick=YY >*/
} CobdfStokesParameter;
extern GType cobdf_stokes_parameter_enum_get_type(void);
#define COBDF_STOKES_PARAMETER_ENUM_TYPE cobdf_stokes_parameter_enum_get_type()

typedef guint64 cobdf_actual_times_t;
typedef guint64 cobdf_actual_durations_t;
typedef guint32 cobdf_flags_t;
typedef gfloat cobdf_zero_lags_t;
typedef float complex cobdf_cross_data_float_t;
typedef INT16_T complex cobdf_cross_data_int16_t;
typedef INT32_T complex cobdf_cross_data_int32_t;
typedef float complex cobdf_auto_data_complex_t;
typedef gfloat cobdf_auto_data_real_t;

G_END_DECLS
#endif /* COBDF_H_ */
