#ifndef LOG_QUEUE_H_
#define LOG_QUEUE_H_

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <glib.h>

G_BEGIN_DECLS

typedef struct _log_queue log_queue_t;
/**
 * log_queue_elem_t:
 * @msg: queue message, can be NULL if no messages are on the queue
 *       (in which case the number of lost messages is noted)
 * @lost: number of messages not pushed onto queue after the previous
 *        message and before the current one
 * @priority: log message priority
 *
 * Element on #log_queue_t queue.
 */
typedef struct {
    gchar *msg;
    guint lost;
    gint priority;
} log_queue_elem_t;

extern log_queue_t *log_queue_new(guint size);
extern log_queue_t* log_queue_ref(log_queue_t *queue);
extern void log_queue_unref(log_queue_t *queue);
extern void log_queue_drop(log_queue_t *queue);
extern void log_queue_accept(log_queue_t *queue);
extern void log_queue_wait_until_empty(log_queue_t *queue);
extern gboolean log_queue_push(log_queue_t *queue, int priority, gchar *msg);
extern log_queue_elem_t *log_queue_pop(log_queue_t *queue);
extern log_queue_elem_t *log_queue_try_pop(log_queue_t *queue);
extern log_queue_elem_t *log_queue_timed_pop(log_queue_t *queue,
                                             GTimeVal *end_time);
extern void log_queue_elem_free(log_queue_elem_t *elem);

G_END_DECLS

#endif /* LOG_QUEUE_H_ */
