/*
 * mdarray
 *
 * Multidimensional arrays with variable number and size of dimensions
 *
 */
#ifndef MDARRAY_H_
#define MDARRAY_H_

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <stdio.h>
#include <sys/mman.h>

G_BEGIN_DECLS

#define COORD_VECTOR_END ((guint)-1)

typedef struct mdarray_t {
    gpointer buffer;
    FILE *stream;
    off_t offset;
    GArray *dimensions;
    gsize element_size;
} mdarray_t;

typedef gboolean (*mdarray_fn_t)(guint *coords, gpointer value,
                                 gpointer user_data);
extern guint64 mdarray_count(const mdarray_t *mdarray);
extern mdarray_t *mdarray_new(const guint *dimensions, gsize element_size);
extern void mdarray_assign_memory(mdarray_t *mdarray, gpointer buffer,
                                  off_t offset);
extern void mdarray_assign_stream(mdarray_t *mdarray, FILE *stream,
                                  off_t offset);
extern void mdarray_unassign(mdarray_t *mdarray);
extern void mdarray_foreach(mdarray_t *mdarray, mdarray_fn_t fn,
                            gpointer user_data);
extern void mdarray_free(mdarray_t *mdarray);
extern off_t mdarray_index_offset(const mdarray_t *mdarray,
                                  const guint *coordinates);
extern mdarray_t *mdarray_reduced(const mdarray_t *mdarray, const guint *fixed,
                                  off_t *element_offset);

static inline guint64 mdarray_size(const mdarray_t *mdarray)
{
    return mdarray_count(mdarray) * mdarray->element_size;
}

G_END_DECLS
#endif /* MDARRAY_H_ */
