#ifndef SDMHDR_H_
#define SDMHDR_H_

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <glib.h>
#include <cobdf.h>

#include <unistd.h>

G_BEGIN_DECLS

typedef enum sdmhdr_xpp_t {
    SDMHDR_XPP_INVALID = -2,
    SDMHDR_XPP_NONE = -1,
    SDMHDR_XPP_RR = 0,
    SDMHDR_XPP_LL,
    SDMHDR_XPP_RR_LL,
    SDMHDR_XPP_RR_RL_LR_LL,
    SDMHDR_XPP_NUM_VALS
} sdmhdr_xpp_t;

typedef enum sdmhdr_app_t {
    SDMHDR_APP_INVALID = -2,
    SDMHDR_APP_NONE = -1,
    SDMHDR_APP_RR = 0,
    SDMHDR_APP_LL,
    SDMHDR_APP_RR_LL,
    SDMHDR_APP_RR_RL_LL,
    SDMHDR_APP_NUM_VALS
} sdmhdr_app_t;

/**
 * sdmhdr_sw_t:
 *
 * Note lack of fields for "scaleFactor" and "sideband" attributes of
 * "spectralWindow" element.
 */
typedef struct sdmhdr_sw_t {
    gchar *id;
    guint num_bin;
    guint num_spectral_point;
    sdmhdr_xpp_t cross_pol_products;
    sdmhdr_app_t sd_pol_products;
} sdmhdr_sw_t;

typedef struct sdmhdr_bb_t {
    guint num_sw;
    sdmhdr_sw_t sw[];
} sdmhdr_bb_t;
#define SIZEOF_SDMHDR_BB(n) (sizeof(sdmhdr_bb_t) + (n) * sizeof(sdmhdr_sw_t))

typedef struct sdmhdr_bincomp_t {
    gsize size;
    gboolean axes[COBDF_NUM_AXES];
} sdmhdr_bincomp_t;

typedef struct sdmhdr_weights_t {
    sdmhdr_bincomp_t common;
    guint num_weights;
    gfloat weights[];
} sdmhdr_weights_t;
#define SIZEOF_SDMHDR_WEIGHTS(n) \
    (sizeof(sdmhdr_weights_t) + (n) * sizeof(gfloat))

typedef struct sdmhdr_dataoid_t {
    gchar *href;
    gchar *title;
    gchar *type;
} sdmhdr_dataoid_t;

typedef struct sdmhdr_execblock_t {
    gchar *href;
    gchar *type;
} sdmhdr_execblock_t;

typedef struct sdmhdr_data_struct_t {
    gchar *type;
    CobdfAtmPhaseCorrection apc[2];
    sdmhdr_bb_t **basebands;
    sdmhdr_bincomp_t *flags;
    sdmhdr_bincomp_t *actual_times;
    sdmhdr_bincomp_t *actual_durations;
    sdmhdr_bincomp_t *zero_lags;
    sdmhdr_bincomp_t *auto_data;
    sdmhdr_bincomp_t *cross_data;
    sdmhdr_weights_t *weights;
} sdmhdr_data_struct_t;

typedef struct sdmhdr_main_hdr_t {
    guint schema_version;
    gchar *revision;
    gchar *main_header_id;
    gchar *project_path;
    CobdfByteOrder byte_order;
    guint64 time;
    guint num_antennas;
    sdmhdr_dataoid_t *data_oid;
    sdmhdr_execblock_t *exec_block;
    CobdfCorrelationMode mode;
    CobdfSpectralResolutionType resolution;
    CobdfAxisName *dimensionality;
    sdmhdr_data_struct_t *data_struct;
} sdmhdr_main_hdr_t;

typedef struct sdmhdr_subset_hdr_t {
    gchar *type;
    gchar *project_path;
    gchar *data_struct_ref;
    guint64 time;
    guint64 interval;
    gchar *flags_href;
    gchar *actual_times_href;
    gchar *actual_durations_href;
    gchar *weights_href;
    gchar *zero_lags_href;
    gchar *cross_data_href;
    gchar *auto_data_href;
    CobdfPrimitiveDataType cross_data_type;
} sdmhdr_subset_hdr_t;

extern sdmhdr_main_hdr_t *sdmhdr_main_hdr_read_memory(
    const gchar *buff, gsize bufflen);
extern sdmhdr_main_hdr_t *sdmhdr_main_hdr_read_stream(FILE *file);
extern sdmhdr_main_hdr_t *sdmhdr_main_hdr_copy(
    const sdmhdr_main_hdr_t *header);
extern gsize sdmhdr_main_hdr_write_memory(const sdmhdr_main_hdr_t *header,
                                          gchar **buff, gboolean indent);
extern gsize sdmhdr_main_hdr_write_stream(const sdmhdr_main_hdr_t *header,
                                          FILE *file, gboolean indent);
extern void sdmhdr_main_hdr_free(sdmhdr_main_hdr_t *sdmhdr);

extern sdmhdr_subset_hdr_t *sdmhdr_subset_hdr_read_memory(
    const gchar *buff, gsize bufflen);
extern sdmhdr_subset_hdr_t *sdmhdr_subset_hdr_read_stream(FILE *file);
extern sdmhdr_subset_hdr_t *sdmhdr_subset_hdr_copy(
    const sdmhdr_subset_hdr_t *header);
extern gsize sdmhdr_subset_hdr_write_memory(
    const sdmhdr_subset_hdr_t *header, gchar **buff, gboolean indent);
extern gsize sdmhdr_subset_hdr_write_stream(const sdmhdr_subset_hdr_t *header,
                                            FILE *file, gboolean indent);
extern void sdmhdr_subset_hdr_free(sdmhdr_subset_hdr_t *header);

extern void sdmhdr_set_flags_size(sdmhdr_main_hdr_t *sdmhdr);
extern void sdmhdr_set_actual_times_size(sdmhdr_main_hdr_t *sdmhdr);
extern void sdmhdr_set_actual_durations_size(sdmhdr_main_hdr_t *sdmhdr);
extern void sdmhdr_set_weights_size(sdmhdr_main_hdr_t *sdmhdr);
extern void sdmhdr_set_zero_lags_size(sdmhdr_main_hdr_t *sdmhdr);
extern void sdmhdr_set_cross_data_size(sdmhdr_main_hdr_t *sdmhdr);
extern void sdmhdr_set_auto_data_size(sdmhdr_main_hdr_t *sdmhdr);

extern void sdmhdr_set_subset_hrefs(const sdmhdr_main_hdr_t *main,
                                    sdmhdr_subset_hdr_t *subset);

inline static guint num_basebands(sdmhdr_bb_t **bb_array)
{
    return g_strv_length((gchar **)bb_array);
}

extern const gchar const *sdmhdr_encoding;

G_END_DECLS

#endif /* SDMHDR_H_ */
