#ifndef TREE_H_
#define TREE_H_

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <glib.h>
#include <sys/mman.h>
#include <stdio.h>
#include <mdarray.h>

G_BEGIN_DECLS

typedef GNode tree_t;
typedef struct root_data_t {
    mdarray_t *mdarray;
    gint mmap_fd;
    gint mmap_prot;
    FILE *stream;
    off_t data_offset;
    gsize element_size;
} root_data_t;
typedef struct branch_t {
    gsize mmap_len;
    gint mmap_prot;
    gint mmap_fd;
    off_t mmap_off;
    gsize mmap_data_offset;
    gpointer mmap_ptr;
    off_t data_offset;
    enum {TREE, ARRAY} type;
    union {
        mdarray_t *array;
        tree_t *tree;
    };
} branch_t;

typedef gboolean (*tree_fn_t)(guint *coordinates, gpointer value,
                               gpointer user_data);

extern tree_t *tree_new(const gchar *spec, gsize element_size);
extern gint tree_height(const tree_t *tree);
extern guint64 tree_count(const tree_t *tree);
extern gint tree_alloc(tree_t *tree);
extern void tree_mmap_pars(tree_t *tree, int fd, int prot, off_t offset);
extern void tree_stream_pars(tree_t *tree, FILE *stream, off_t offset);
extern void tree_free(tree_t *tree);

extern branch_t *tree_branch(const tree_t *tree, const guint *coordinates);
extern branch_t *tree_branch_root(const tree_t *tree);
extern branch_t *tree_branch_branch(const branch_t *branch,
                                    const guint *coordinates);
extern guint64 tree_branch_count(const branch_t *branch);
extern gint tree_branch_height(const branch_t *branch);
extern void tree_branch_clear(branch_t *branch);
extern gsize tree_branch_element_size(const branch_t *branch);
extern void tree_branch_foreach(branch_t *branch, tree_fn_t fn,
                                gpointer user_data);
extern gint tree_branch_mmap(branch_t *branch);
extern void tree_branch_munmap(branch_t *branch);
extern void tree_branch_free(branch_t *branch);
extern FILE *tree_branch_stream(const branch_t *branch);
extern gpointer tree_branch_memory(const branch_t *branch);
extern off_t tree_branch_index_offset(const branch_t *branch,
                                      const guint *coordinates);

static inline gsize tree_size(const tree_t *tree)
{
    root_data_t *rdata = g_node_get_root((tree_t *)tree)->data;
    return rdata->element_size * tree_count(tree);
}

static inline gsize tree_branch_size(const branch_t *branch)
{
    return tree_branch_element_size(branch) * tree_branch_count(branch);
}

G_END_DECLS

#endif /* TREE_H_ */
