#ifndef WCBE_CONFIG_H_
#define WCBE_CONFIG_H_

#include <glib.h>
#include <wcbe_enums.h>
#include <sdmhdr.h>

G_BEGIN_DECLS

typedef struct wcbe_input_config {
    GArray *streams;
    GPtrArray *products;
} wcbe_input_config_t;

typedef struct wcbe_stream_config {
    gchar *id;
    guint station_id;
    guint baseband_id;
    guint subband_id;
} wcbe_stream_config_t;

typedef struct wcbe_integration_config {
    guint min_hw;
    guint hw;
    guint lta;
} wcbe_integration_config_t;

typedef struct wcbe_product_config {
    gchar *id;
    guint num_lags;
    guint num_bins;
    guint subband_index;
    wcbe_stream_config_t *streamA;
    wcbe_stream_config_t *streamB;
    wcbe_integration_config_t integration;
    WcbeCorrBits nbits; // 4 or 7
} wcbe_product_config_t;

typedef struct wcbe_product4_config {
    wcbe_product_config_t common;
    GArray *lag_chains;
} wcbe_product4_config_t;

typedef struct wcbe_product7_config {
    wcbe_product_config_t common;
    GArray *lag_chains_ll;
    GArray *lag_chains_lm;
    GArray *lag_chains_ml;
    GArray *lag_chains_mm;
} wcbe_product7_config_t;

typedef struct wcbe_input_stream_config {
    gchar stream; // 'A' or 'B'
    guint lag_block;
} wcbe_input_stream_config_t;

typedef enum wcbe_7bit_corr_subproduct {
    WCBE_7BIT_LL = 0, WCBE_7BIT_LM = 1, WCBE_7BIT_ML = 2, WCBE_7BIT_MM = 3
} wcbe_7bit_corr_subproduct_t;

typedef struct wcbe_lag_chain_config {
    wcbe_7bit_corr_subproduct_t subprod;
    guint16 board_id;
    guint ccX;
    guint ccY;
    guint ccc0;
    guint cccN;
    guint recirc;
    wcbe_input_stream_config_t inpX;
    wcbe_input_stream_config_t inpY;
} wcbe_lag_chain_config_t;

typedef struct wcbe_output_config {
    GArray *subarrays;
} wcbe_output_config_t;

typedef struct wcbe_subarray_config {
    GArray *stations;
    guint num_apc;
    guint integration;
    GArray *basebands;
} wcbe_subarray_config_t;

typedef struct wcbe_baseband_config {
    GArray *spectral_windows;
    guint num_polarizations;
    gint baseband_ids[2]; // (left, right) or (primary, secondary)
} wcbe_baseband_config_t;

typedef struct wcbe_spectral_window_config {
    guint num_subbands;
    guint num_bins;
    guint num_channels;
    sdmhdr_xpp_t xpp;
    sdmhdr_app_t sdpp;
} wcbe_spectral_window_config_t;

typedef struct wcbe_pipelines_config {
    GArray *pipelines;
} wcbe_pipelines_config_t;

typedef struct wcbe_pipeline_config {
    gchar *id;
    guint subarray_index;
    union {
        char *gst_spec;
    } spec;
    GArray *swmaps;
} wcbe_pipeline_config_t;

typedef struct wcbe_swmap_config {
    guint baseband_index;
    guint spectral_window_index;
    GArray *subband_indices;
} wcbe_swmap_config_t;

typedef struct wcbe_config {
    gchar *name;
    gchar *dir;
    wcbe_input_config_t input;
    wcbe_output_config_t output;
    wcbe_pipelines_config_t pipelines;
} wcbe_config_t;

extern gboolean wcbe_config_parse(const gchar *doc, wcbe_config_t *config);
extern void wcbe_config_free(wcbe_config_t *config);
extern void wcbe_config_doc(const wcbe_config_t *config, gchar **buf);

G_END_DECLS

#endif /* WCBE_CONFIG_H_ */
