#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <wcbe_lagset.h>
#include <string.h>

#if WCBE_LAGSET_HEADER_BLOCK_INDEX != 0 \
    || WCBE_LAGSET_FRAME_HEADERS_BLOCK_INDEX != 1 \
    || WCBE_LAGSET_LAGS_BLOCK_INDEX != 2 \
    || WCBE_LAGSET_NUM_BLOCKS != 3
# error "Lagset vector block index macros have unsupported values"
#endif

static void wcbe_lagset_init_hdr(wcbe_vect_t *lagset, guint num_frames_per_bin,
                                 guint num_lags_per_bin, guint num_bins,
                                 WcbeCorrBits nbits);

wcbe_vect_t *
wcbe_lagset_new(guint num_frames_per_bin, guint num_lags_per_bin,
                guint num_bins, WcbeCorrBits nbits, gboolean zero_mem)
{
    g_return_val_if_fail(nbits == WCBE_4BITS || nbits == WCBE_7BITS, NULL);

    guint nbits_multiplier = (nbits == WCBE_7BITS ? 4 : 1);
    wcbe_vect_t *result = wcbe_vect_alloc(
        WCBE_LAGSET_NUM_BLOCKS,
        1, sizeof(wcbe_lagset_header_t),
        num_frames_per_bin * num_bins * nbits_multiplier,
        sizeof(wcbe_lagset_frame_header_t),
        num_lags_per_bin * num_bins * nbits_multiplier, sizeof(lf_lag_t));
    if (zero_mem) {
        wcbe_vect_clear_all(result);
    }
    wcbe_lagset_init_hdr(result, num_frames_per_bin, num_lags_per_bin,
                         num_bins, nbits);
    return result;
}

gsize
wcbe_lagset_alloc_size(guint num_frames_per_bin, guint num_lags_per_bin,
                       guint num_bins, WcbeCorrBits nbits)
{
    g_return_val_if_fail(nbits == WCBE_4BITS || nbits == WCBE_7BITS, 0);

    guint nbits_multiplier = (nbits == WCBE_7BITS ? 4 : 1);
    return wcbe_vect_alloc_size(
        WCBE_LAGSET_NUM_BLOCKS,
        1, sizeof(wcbe_lagset_header_t),
        num_frames_per_bin * num_bins * nbits_multiplier,
        sizeof(wcbe_lagset_frame_header_t),
        num_lags_per_bin * num_bins * nbits_multiplier, sizeof(lf_lag_t));
}

void
wcbe_lagset_init(guint num_frames_per_bin, guint num_lags_per_bin,
                 guint num_bins, WcbeCorrBits nbits, gboolean zero_mem,
                 wcbe_vect_t *lagset)
{
    g_return_if_fail(nbits == WCBE_4BITS || nbits == WCBE_7BITS);

    guint nbits_multiplier = (nbits == WCBE_7BITS ? 4 : 1);
    wcbe_vect_init_from_pars(
        lagset,
        WCBE_LAGSET_NUM_BLOCKS,
        1, sizeof(wcbe_lagset_header_t),
        num_frames_per_bin * num_bins * nbits_multiplier,
        sizeof(wcbe_lagset_frame_header_t),
        num_lags_per_bin * num_bins * nbits_multiplier, sizeof(lf_lag_t));
    if (zero_mem) {
        wcbe_vect_clear_all(lagset);
    }
    wcbe_lagset_init_hdr(lagset, num_frames_per_bin, num_lags_per_bin,
                         num_bins, nbits);
}

static void
wcbe_lagset_init_hdr(wcbe_vect_t *lagset, guint num_frames_per_bin,
                     guint num_lags_per_bin, guint num_bins,
                     WcbeCorrBits nbits)
{
    g_return_if_fail(nbits == WCBE_4BITS || nbits == WCBE_7BITS);
    g_return_if_fail((num_lags_per_bin / num_frames_per_bin == 128)
                     || (num_lags_per_bin / num_frames_per_bin == 64));

    wcbe_lagset_header_t *hdr =
        &wcbe_vect_index(lagset, WCBE_LAGSET_HEADER_BLOCK_INDEX,
                         wcbe_lagset_header_t, 0);
    hdr->num_frames_per_bin = num_frames_per_bin;
    hdr->num_lags_per_bin = num_lags_per_bin;
    hdr->num_bins = num_bins;
    hdr->nbits = nbits;
}
