#ifndef WCBE_LAGSET_H_
#define WCBE_LAGSET_H_

#include <glib.h>
#include <wcbe_util.h>
#include <wcbe_lag_frame.h>
#include <wcbe_config.h>
#include <wcbe_enums.h>

#define WCBE_LAGSET_MEDIA_TYPE "application/x.wcbe.lagset"

G_BEGIN_DECLS

typedef struct wcbe_lagset_frame_header_t {
    lf_data_frame_word_t key[LF_KEY_LENGTH];
    gboolean reverse_conjugate;
    gboolean filled;
    lf_widar_time_t timestamp;
    guint32 data_valid_center;
    guint32 data_valid_edge;
    guint32 data_bias; //used only by CC lag frames
    guint32 checksum;
} wcbe_lagset_frame_header_t;

#define HEADER_START_BLOCKY(h) (LF_BITFIELD_EXTRACT((h)->key[0], 8, 24))
#define HEADER_NBLOCKS(h) (LF_BITFIELD_EXTRACT((h)->key[0], 8, 16))
#define HEADER_NUM_LAGS(h) (128 >> (LF_BITFIELD_EXTRACT((h)->key[0], 13, 3)))
#define HEADER_CHIPID(h) (LF_BITFIELD_EXTRACT((h)->key[0], 6, 7))
#define HEADER_CCC(h) (LF_BITFIELD_EXTRACT((h)->key[0], 4, 3))
#define HEADER_REV_CONJ(h) (LF_BITFIELD_EXTRACT((h)->key[0], 3, 0))

#define HEADER_BBID_B(h) (LF_BITFIELD_EXTRACT((h)->key[1], 3, 29))
#define HEADER_SBID_B(h) (LF_BITFIELD_EXTRACT((h)->key[1], 5, 24))
#define HEADER_SID_B(h) (LF_BITFIELD_EXTRACT((h)->key[1], 8, 16))
#define HEADER_BBID_A(h) (LF_BITFIELD_EXTRACT((h)->key[1], 3, 13))
#define HEADER_SBID_A(h) (LF_BITFIELD_EXTRACT((h)->key[1], 5, 8))
#define HEADER_SID_A(h) (LF_BITFIELD_EXTRACT((h)->key[1], 8, 0))

#define HEADER_STATUS_BITS(h) (LF_BITFIELD_EXTRACT((h)->key[2], 7, 25))
#define HEADER_FRAME_COUNT(h) (LF_BITFIELD_EXTRACT((h)->key[2], 9, 16))
#define HEADER_RECIRC_BLK_B(h) (LF_BITFIELD_EXTRACT((h)->key[2], 8, 8))
#define HEADER_RECIRC_BLK_A(h) (LF_BITFIELD_EXTRACT((h)->key[2], 8, 0))

#define HEADER_BOARD_SN(h) (LF_BITFIELD_EXTRACT((h)->key[3], 16, 16))
#define HEADER_LTA_BIN(h) (LF_BITFIELD_EXTRACT((h)->key[3], 16, 0))

typedef struct wcbe_lagset_header_t {
    GQuark config;
    GQuark id;
    lf_widar_time_t timestamp;
    guint num_frames_per_bin; // number of frames per bin (w/o accounting
                              // for multiplicity due to 7-bit
                              // correlation)
    guint num_lags_per_bin;   // number of (complex) lags per bin (w/o
                              // accounting for multiplicity due to
                              // 7-bit correlation)
    guint num_bins;   // number of bins (multiple of lag sets)
    WcbeCorrBits nbits;
    guint stnA;
    guint stnB;
    guint bbA;
    guint bbB;
    guint sbIdx;          // (renumbered) sub-band index
} wcbe_lagset_header_t;

#define WCBE_LAGSET_HEADER_BLOCK_INDEX 0
#define WCBE_LAGSET_FRAME_HEADERS_BLOCK_INDEX 1
#define WCBE_LAGSET_LAGS_BLOCK_INDEX 2
#define WCBE_LAGSET_NUM_BLOCKS 3
#define WCBE_LAGSET_LAG_4BIT_INDEX(hdr, lagnum, binnum) \
    ((binnum) * (hdr)->num_lags_per_bin + (lagnum))
#define WCBE_LAGSET_SUBPROD_MM WCBE_7BIT_MM
#define WCBE_LAGSET_SUBPROD_ML WCBE_7BIT_ML
#define WCBE_LAGSET_SUBPROD_LM WCBE_7BIT_LM
#define WCBE_LAGSET_SUBPROD_LL WCBE_7BIT_LL
#define WCBE_LAGSET_LAG_7BIT_INDEX(hdr, lagnum, binnum, subprd) \
    ((hdr)->num_lags_per_bin * (4 * (binnum) + (subprd)) + (lagnum))
#define WCBE_LAGSET_LAG_TO_FRAME_HEADER_INDEX(hdr, lagidx) \
    ((lagidx) / (hdr)->lags_per_frame)
#define WCBE_LAGSET_FRAME_HEADER_TO_FIRST_LAG_INDEX(hdr, frameidx) \
    ((frameidx) * (hdr)->lags_per_frame)

extern wcbe_vect_t *wcbe_lagset_new(guint num_frames_per_bin,
                                    guint num_lags_per_bin,
                                    guint num_bins, WcbeCorrBits nbits,
                                    gboolean zero_mem);
extern gsize wcbe_lagset_alloc_size(guint num_frames_per_bin,
                                    guint num_lags_per_bin,
                                    guint num_bins, WcbeCorrBits nbits);
extern void wcbe_lagset_init(guint num_frames_per_bin, guint num_lags_per_bin,
                             guint num_bins, WcbeCorrBits nbits,
                             gboolean zero_mem, wcbe_vect_t *lagset);

G_END_DECLS

#endif /* WCBE_LAGSET_H_ */
