#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <gst/gst.h>
#include <wcbe_lagset_src.h>
#include <wcbe_lagset_demux.h>
#include <wcbe_accumulate.h>
#include <wcbe_fdvc_normalize.h>
#include <wcbe_lc7.h>
#include <wcbe_vect_rotate.h>
#include <wcbe_fft.h>
#include <wcbe_bdf_visdata.h>

struct _elements_entry {
    const gchar *name;
    guint rank;
    GType (*type) (void);
};

static struct _elements_entry _elements[] = {
    {"wcbe_lagset_src", GST_RANK_NONE, wcbe_lagset_src_get_type},
    {"wcbe_lagset_demux", GST_RANK_NONE, wcbe_lagset_demux_get_type},
    {"wcbe_accumulate", GST_RANK_NONE, wcbe_accumulate_get_type},
    {"wcbe_fdvc_normalize", GST_RANK_NONE, wcbe_fdvc_normalize_get_type},
    {"wcbe_lc7", GST_RANK_NONE, wcbe_lc7_get_type},
    {"wcbe_vect_rotate", GST_RANK_NONE, wcbe_vect_rotate_get_type},
    {"wcbe_fft", GST_RANK_NONE, wcbe_fft_get_type},
    {"wcbe_bdf_visdata", GST_RANK_NONE, wcbe_bdf_visdata_get_type},
    {NULL, 0},
};

static gboolean
plugin_init (GstPlugin *plugin)
{
    struct _elements_entry *element = _elements;

    while (element->name != NULL) {
        if (!gst_element_register(plugin, element->name, element->rank,
                                  element->type())) {
            return FALSE;
        }
        element++;
    }
    return TRUE;
}

GST_PLUGIN_DEFINE (
    GST_VERSION_MAJOR,
    GST_VERSION_MINOR,
    "wcbecoreelements",
    "WIDAR backend lag processing stage core elements",
    plugin_init,
    VERSION,
    "GPL",
    PACKAGE,
    "http://www.aoc.nrao.edu/evla"
    );
