#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <gst/gst.h>
#include <wcbe_src.h>
#include <wcbe_framedump.h>
#include <wcbe_decode.h>
#include <wcbe_sort.h>
#include <wcbe_lagset_sink.h>
#include <wcbe_lagset_dump.h>

struct _elements_entry {
    const gchar *name;
    guint rank;
    GType (*type) (void);
};

static struct _elements_entry _elements[] = {
    {"wcbe_src", GST_RANK_NONE, wcbe_src_get_type},
    {"wcbe_framedump", GST_RANK_NONE, wcbe_framedump_get_type},
    {"wcbe_decode", GST_RANK_NONE, wcbe_decode_get_type},
    {"wcbe_sort", GST_RANK_NONE, wcbe_sort_get_type},
    {"wcbe_lagset_dump", GST_RANK_NONE, wcbe_lagset_dump_get_type},
    {"wcbe_lagset_sink", GST_RANK_NONE, wcbe_lagset_sink_get_type},
    {NULL, 0},
};

static gboolean
plugin_init (GstPlugin *plugin)
{
    struct _elements_entry *element = _elements;

    while (element->name != NULL) {
        if (!gst_element_register(plugin, element->name, element->rank,
                                  element->type())) {
            return FALSE;
        }
        element++;
    }
    return TRUE;
}

GST_PLUGIN_DEFINE (
    GST_VERSION_MAJOR,
    GST_VERSION_MINOR,
    "wcbeinputstage",
    "WIDAR backend input stage elements",
    plugin_init,
    VERSION,
    "GPL",
    PACKAGE,
    "http://www.aoc.nrao.edu/evla"
    );
