import threading

class Dict (dict):
    'Thread-safe dictionary'

    def __init__(self):
        super(Dict, self).__init__()
        self.lock = threading.RLock()
        return

    def acquire(self):
        self.lock.acquire()
        return

    def release(self):
        self.lock.release()
        return

    def __len__(self):
        self.lock.acquire()
        try:
            result = super(Dict, self).__len__()
        finally:
            self.lock.release()
        return result

    def __getitem__(self, key):
        self.lock.acquire()
        try:
            result = super(Dict, self).__getitem__(key)
        finally:
            self.lock.release()
        return result

    def __setitem__(self, key, value):
        self.lock.acquire()
        try:
            super(Dict, self).__setitem__(key, value)
        finally:
            self.lock.release()
        return

    def __delitem__(self, key):
        self.lock.acquire()
        try:
            super(Dict, self).__delitem__(key)
        finally:
            self.lock.release()
        return

    def __iter__(self):
        self.lock.acquire()
        try:
            cpy = {}
            for (k, v) in super(Dict, self).items():
                cpy[k] = v
        finally:
            self.lock.release()
        return cpy.__iter__()

    def __contains__(self, key):
        self.lock.acquire()
        try:
            result = super(Dict, self).__contains__(key)
        finally:
            self.lock.release()
        return result
